/*! 
	\file un.c
	\author GOIGOUX Lucie & JEUDI--LEMOINE Alix
	\date 22/12/22
	\brief Partie 1 de la SAE 1.02

	Application de gestion des candidature dans les IUT de France
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h> // sleep
#include <strings.h> // strcasecmp : insensitive case
#include "../II/deux.c"
#include "un.h"

void identificationCandidat(int* id, ListeCandidats* listePointer, int* nbCandidats) {
	char connexion;
	
	printf("\nÊtes-vous déjà enregistré (O/N) ? : ");
	scanf("%*c%c", &connexion);

	char nom[LEN_MAX];

	if(connexion == 'O') {
		printf("\nTrès bien, merci de saisir votre ID : ");
		scanf("%d", id);

		if(*id > *nbCandidats) {
			*id = 0;
			fprintf(stderr, "\n\e[0;91mErreur : cet ID n'existe pas !\n\n\e[0m");
		}

		printf("Merci de confirmer votre nom de famille : ");
		scanf("%s", nom);

		if(strcasecmp(nom, (*listePointer)[*id-1]->nom) != 0) {
			fprintf(stderr, "\n\e[0;91mErreur : le nom de famille ne correspond pas !\n\n\e[0m");
		} else {
			printf("\n\e[1;92mBienvenue %s %s !\n\e[1;91m"
				   "N'oubliez pas de revenir au menu principal pour vous déconnecter à la fin de votre session !\n\n\e[0m",
				   (*listePointer)[*id-1]->prenom, (*listePointer)[*id-1]->nom);
		}
	} else {
		printf("\nTrès bien, nous allons donc précéder à votre enregistrement.\n\n");

		Candidat* candidat = creerCandidat(); // malloc

		printf("Nom de famille : ");
		scanf("%*c%[^\n]", candidat->nom);
		printf("Prénom : ");
		scanf("%*c%[^\n]", candidat->prenom);
		printf("Moyenne en maths : ");
		scanf("%f", &candidat->moyenneMath);
		printf("Moyenne en français : ");
		scanf("%f", &candidat->moyenneFrancais);
		printf("Moyenne en anglais : ");
		scanf("%f", &candidat->moyenneAnglais);
		printf("Moyenne en spécialité : ");
		scanf("%f", &candidat->moyenneSpecialite);

		ajouterCandidat(listePointer, nbCandidats, candidat); // realloc
		*id = candidat->id;

		printf("\n\n\e[1mParfait ! Votre ID est \e[1;32m%d\e[0m\e[1m, ne le perdez pas, "
			   "il vous suivera tout le long de vos candidatures !\n\e[1;91m"
			   "N'oubliez pas de revenir au menu principal pour vous déconnecter à la fin de votre session !\n\n\e[0m", candidat->id);
	}
}

void titreMenuPrincipal(void) {
	system("clear");
	printf("\e[1;35m██████╗ ███████╗ ██████╗██████╗ ██╗   ██╗████████╗███████╗███╗   ███╗███████╗███╗   ██╗████████╗    ██╗██╗   ██╗████████╗\n");
	printf("██╔══██╗██╔════╝██╔════╝██╔══██╗██║   ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝████╗  ██║╚══██╔══╝    ██║██║   ██║╚══██╔══╝\n");
	printf("██████╔╝█████╗  ██║     ██████╔╝██║   ██║   ██║   █████╗  ██╔████╔██║█████╗  ██╔██╗ ██║   ██║       ██║██║   ██║   ██║   \n");
	printf("██╔══██╗██╔══╝  ██║     ██╔══██╗██║   ██║   ██║   ██╔══╝  ██║╚██╔╝██║██╔══╝  ██║╚██╗██║   ██║       ██║██║   ██║   ██║   \n");
	printf("██║  ██║███████╗╚██████╗██║  ██║╚██████╔╝   ██║   ███████╗██║ ╚═╝ ██║███████╗██║ ╚████║   ██║       ██║╚██████╔╝   ██║   \n");
	printf("╚═╝  ╚═╝╚══════╝ ╚═════╝╚═╝  ╚═╝ ╚═════╝    ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝   ╚═╝       ╚═╝ ╚═════╝    ╚═╝   \e[0m\n\n");
}

void menuCandidat(VilleIUT*** tiutPointer, ListeCandidats* listePointer, int* nbVilles, int* nbCandidats) {
	char poubelle; // Permet de stocker les caractères aspirés en trop sur stdin lors du choix dans le menu
	int choix, id = 0; // choix: Contient le choix de l'utilisateur dans le menu / id: contient id user si login
	char connexion;

	printf("\n\nSouhaitez-vous vous connecter/enregistrer maintenant ou entrer en mode invité (O/N) ? : ");
	scanf("%*c%c", &connexion);

	if(connexion == 'O')
		identificationCandidat(&id, listePointer, nbCandidats);

	if(id != 0)
		printf("\e[1;94mCandidat: %s %s, ID: %d\e[0m\n", (*listePointer)[id-1]->prenom, (*listePointer)[id-1]->nom, id);
	else
		printf("\n\e[1;94mInvité\e[0m\n");
	printf("-----------------------------------------------------------------------\n");
	printf("\t1 - Consulter les ville possédant un IUT\n\n");
	printf("\t2 - Voir les disponibilités des départements dans un IUT\n\n");
	printf("\t3 - Rechercher un département\n\n");
	if(id != 0) {
		printf("\t4 - Candidater\n\n");
		printf("\t5 - Suivre sa candidature\n\n");	
	} else {
		printf("\t4 - S'identifier\n\n");
	}
	printf("\t9 - Quitter\n");
	printf("-----------------------------------------------------------------------\n");


	if(scanf("%d", &choix) == 0)
		choix=0;

	printf("-----------------------------------------------------------------------\n");
	
	while(choix != 9){
		switch (choix) {
			case 1:
				//system("clear");
				printf("Consulter les ville possédant un IUT...\n\n");
                afficherListeVilles(*tiutPointer, *nbVilles);
                break;
			case 2:
                printf("Voir les disponibilités des départements dans un IUT...\n\n");
				printf("Saisissez le nom de la ville recherchée (tapez q pour annuler) : ");
				
				char searchIUT[LEN_MAX];
				scanf("%s", searchIUT);

				if(strcasecmp(searchIUT, "q") == 0)
					break;

				afficherListeDepartement(*tiutPointer, *nbVilles, searchIUT);
				break;
			case 3:
				printf("Rechercher un département...\n\n");
				printf("Saisissez le nom du département recherché (tapez q pour annuler) : ");

				char searchDept[LEN_MAX];
				scanf("%*c%[^\n]", searchDept);
				
				if(strcasecmp(searchDept, "q") == 0)
					break;

				afficherDepartement(*tiutPointer, *nbVilles, searchDept);
				break;
			case 4:
				if(id != 0) {
					printf("Candidater...\n\n");
				} else {
					printf("S'identifier...\n\n");
					identificationCandidat(&id, listePointer, nbCandidats);
				}
					
				break;
			case 5:
				printf("Suivre sa candidature...\n\n");
				if(id == 0) {
					fprintf(stderr, "\e[0;91mErreur : vous devez d'abord vous authentifier, merci de réessayer plus tard.\n\e[0m");
				}
				// TODO
				break;
			default:
				// Permet d'aspirer les caractères en trop sur stdin (évite de répéter le message d'erreur X fois)
				while ((poubelle = getchar()) != '\n' && poubelle != EOF);
				fprintf(stderr, "\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer.\n\e[0m");
				break;
		}

		if(id != 0)
			printf("\n\e[1;94mCandidat: %s %s, ID: %d\e[0m\n", (*listePointer)[id-1]->prenom, (*listePointer)[id-1]->nom, id);
		else
			printf("\n\e[1;94mInvité\e[0m\n");
		printf("-----------------------------------------------------------------------\n");
		printf("\t1 - Consulter les ville possédant un IUT\n\n");
		printf("\t2 - Voir les disponibilités des départements dans un IUT\n\n");
		printf("\t3 - Rechercher un département\n\n");
		if(id != 0) {
			printf("\t4 - Candidater\n\n");
			printf("\t5 - Suivre sa candidature\n\n");	
		} else {
			printf("\t4 - S'identifier\n\n");
		}
		printf("\t9 - Quitter\n");
		printf("-----------------------------------------------------------------------\n");
		
		if(scanf("%d", &choix) == 0)
			choix=0;
	}

	printf("\nRetour au menu principal...\n");
}

void menuResponsable(VilleIUT*** tiutPointer, ListeCandidats* listePointer, int* nbVilles, int* nbCandidats) {
	//VilleIUT** tiut = *tiutPointer;
	char poubelle; // Permet de stocker les caractères aspirés en trop sur stdin lors du choix dans le menu
	int choix;

	printf("--------------------------------------------------------------------\n");
	printf("\t1 - Lister les candidatures\n\n");
	printf("\t2 - Traiter les candidatures\n\n");
	printf("\t9 - Quitter\n");
	printf("---------------------------------------------------------------------\n");

	if(scanf("%d", &choix) == 0)
		choix=0;

	printf("-----------------------------------------------------------------------\n");
	
	while(choix != 9){
		switch (choix) {
			case 1:
                printf("Lister les candidatures...\n");
				// TODO
				break;
			case 2:
				printf("Traiter les candidatures...\n");
				// TODO
				break;
			default:
				// Permet d'aspirer les caractères en trop sur stdin (évite de répéter le message d'erreur X fois)
				while ((poubelle = getchar()) != '\n' && poubelle != EOF);
				fprintf(stderr, "\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer.\n\e[0m");
				break;
		}

		printf("\n--------------------------------------------------------------------\n");
		printf("\t1 - Lister les candidatures\n\n");
		printf("\t2 - Traiter les candidatures\n\n");
		printf("\t9 - Quitter\n");
		printf("---------------------------------------------------------------------\n");
		
		if(scanf("%d", &choix) == 0)
			choix=0;
	}

	printf("\nRetour au menu principal...\n");
}

void menuAdmin(VilleIUT*** tiutPointer, ListeCandidats* listePointer, int* nbVilles, int* nbCandidats) {
	char poubelle; // Permet de stocker les caractères aspirés en trop sur stdin lors du choix dans le menu
	int choix; // Contient le choix de l'utilisateur dans le menu
	char pass[LEN_MAX];

	printf("Merci d'entrer le mot de passe administrateur : ");
	system("stty -echo");
	scanf("%s", pass);
	system("stty echo");

	if(strcmp(pass, "admin2023") != 0) {
		fprintf(stderr, "\n\e[1;91mErreur: mot de passe incorrect !\n\n\e[0m");
		sleep(2);
		return;
	} else
		printf("\n\n\e[1;32mMot de passe correct !\e[0m\n\n");

	printf("--------------------------------------------------------------------\n");
	printf("\t1 - Modifier le nombre de place d'une formation\n\n");
	printf("\t2 - Créer un nouveau département\n\n");
	printf("\t3 - Supprimer un département\n\n");
	printf("\t4 - Lancer/arrêter la phase de candidature\n\n");
	printf("\t5 - Modifier le nom du responsable d'un département\n\n");
	printf("\t6 - Liste des candidats\n\n");
	printf("\t9 - Quitter\n");
	printf("---------------------------------------------------------------------\n");

	if(scanf("%d", &choix) == 0)
		choix=0;

	printf("---------------------------------------------------------------------\n");
	
	while(choix != 9){
		switch (choix) {
			case 1:
				printf("Modifier le nombre de place d'une formation...\n\n");

				int nbPlaces;
				char searchIUT[LEN_MAX], searchDept[LEN_MAX];
				printf("Entrez une ville (tapez q pour annuler) : ");
				scanf("%s", searchIUT);

				if(strcasecmp(searchIUT, "q") == 0)
					break;

				int code = afficherListeDepartement(*tiutPointer, *nbVilles, searchIUT);

				if(code) {
					printf("Quel département souhaitez-vous modifier ? : ");
					scanf("%s", searchDept);
					printf("Entrez le nouveau nombre de places pour le département '%s' : ", searchDept);
					scanf("%d", &nbPlaces);

					modifierNbPlaces(*tiutPointer, *nbVilles, searchIUT, searchDept, nbPlaces);
				}

                break;
			case 2:
                printf("Créer un nouveau département...\n\n");
			
				char searchIUT2[LEN_MAX];
				printf("Entrez la ville dans laquelle vous souhaitez ajouter un département (tapez q pour annuler) : ");
				scanf("%s", searchIUT2);

				if(strcasecmp(searchIUT2, "q") == 0)
					break;

				int i2 = rechercherVille(*tiutPointer, *nbVilles, searchIUT2);

				if(i2 == -1)
					fprintf(stderr, "\n\e[1;91mErreur: la ville '%s' n'est pas dans la liste des IUT.\e[0m\n\n", searchIUT2);
				else {
					VilleIUT* ville = *tiutPointer[i2];

					int nbP;
					char addDept[LEN_MAX], responsable[LEN_MAX];

					printf("Entrez le nom du nouveau département : ");
					scanf("%*c%[^\n]", addDept);

					ListeDept ldept = ville->ldept;

					if(strcmp(ldept->departement, addDept) == 0) {
						fprintf(stderr, "\n\e[1;91mErreur: le département '%s' existe déjà dans l'IUT '%s'.\e[0m\n\n", addDept, searchIUT2);	
						break;
					}

					while(ldept->suiv != NULL) {
						if(strcmp(ldept->suiv->departement, addDept) == 0) break;
						ldept = ldept->suiv;
					}

					if(ldept->suiv != NULL && strcmp(ldept->suiv->departement, addDept) == 0) {
						fprintf(stderr, "\n\e[1;91mErreur: le département '%s' existe déjà dans l'IUT '%s'.\e[0m\n\n", addDept, searchIUT2);	
						break;
					}

					printf("Entrez le nombre de place disponibles en 1ère année : ");
					scanf("%d", &nbP);
					printf("Entrez le nom du responsable de ce département : ");
					scanf("%*c%[^\n]", responsable);

					MaillonDept* dept = creerDepartement(addDept, nbP, responsable);
					ville->ldept = ajouterDepartement(ville->ldept, dept);

					printf("\n\e[1;32mLe département '%s' a bien été ajouté à l'IUT '%s' !\e[0m\n\n", addDept, searchIUT2);
				}
				break;
			case 3:
				printf("Supprimer un département...\n\n");

				char searchIUT3[LEN_MAX];
				printf("Entrez la ville dans laquelle vous souhaitez supprimer un département (tapez q pour annuler) : ");
				scanf("%s", searchIUT3);

				if(strcasecmp(searchIUT3, "q") == 0)
					break;

				int i3 = rechercherVille(*tiutPointer, *nbVilles, searchIUT3);

				if(i3 == -1)
					fprintf(stderr, "\e[1;91mErreur: la ville '%s' n'est pas dans la liste des IUT.\e[0m\n\n", searchIUT3);
				else {
					VilleIUT* ville3 = *tiutPointer[i3];

					afficherListeDepartement(*tiutPointer, *nbVilles, searchIUT3);

					char searchDept3[LEN_MAX];
					printf("Quel département souhaitez-vous supprimer ? : ");
					scanf("%*c%[^\n]", searchDept3);

					ville3->ldept = supprimerDepartement(ville3->ldept, searchDept3);

					
				}
				break;
			case 4:
				printf("Lancer/arrêter la phase de candidature...\n\n");
				// TODO
				break;
			case 5:
				printf("Modifier le nom du responsable d'un département...\n\n");

				char searchIUT5[LEN_MAX], searchDept5[LEN_MAX], nom[LEN_MAX];
				printf("Dans quel IUT souhaitez-vous faire une modification ? (tapez q pour annuler) : ");
				scanf("%s", searchIUT5);

				if(strcasecmp(searchIUT5, "q") == 0)
					break;
	
				int code5 = afficherListeDepartement(*tiutPointer, *nbVilles, searchIUT5);

				if(code5 != -1) {
					printf("Quel département souhaitez-vous modifier ? : ");
					scanf("%s", searchDept5);
					
					printf("Quel est le nom du nouveau responsable : ");
					scanf("%*c%[^\n]", nom);

					modifierNomResponsable(*tiutPointer, *nbVilles, searchIUT5, searchDept5, nom);
				}
				break;
			case 6:
				printf("Liste des candidats...\n");
				printf("\n\e[4;37mListe des candidats (triés par ordre alphabétique) :\e[0m\n\n");
                afficherListeCandidats(*listePointer, *nbCandidats);
				break;
			default:
				// Permet d'aspirer les caractères en trop sur stdin (évite de répéter le message d'erreur X fois)
				while ((poubelle = getchar()) != '\n' && poubelle != EOF);
				fprintf(stderr, "\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer.\n\e[0m");
				break;
		}

		printf("\n--------------------------------------------------------------------\n");
		printf("\t1 - Modifier le nombre de place d'une formation\n\n");
		printf("\t2 - Créer un nouveau département\n\n");
		printf("\t3 - Supprimer un département\n\n");
		printf("\t4 - Lancer/arrêter la phase de candidature\n\n");
		printf("\t5 - Modifier le nom du responsable d'un département\n\n");
		printf("\t6 - Liste des candidats\n\n");
		printf("\t9 - Quitter\n");
		printf("---------------------------------------------------------------------\n");
		
		if(scanf("%d", &choix) == 0)
			choix=0;
	}

	printf("\nRetour au menu principal...\n");
}

void menuPrincipal(/* TODO: prise en compte des status actuels du recrutement */) {
	char poubelle; // Permet de stocker les caractères aspirés en trop sur stdin lors du choix dans le menu
    int choix; // Contient le choix de l'utilisateur dans le menu

	// PARTIE 1
	int nbVilles = 0;
	VilleIUT** tiut = readVilles(&nbVilles);

	// PARTIE 2
	int nbCandidats = 0;
	ListeCandidats liste = readCandidats(&nbCandidats);

	titreMenuPrincipal();
	printf("--------------------------------------------\n");
	printf("Vous êtes : \n");
	printf("--------------------------------------------\n");
    printf("\t1 - Candidat\n\n");
    printf("\t2 - Administrateur\n\n");
	printf("\t3 - Responsable\n\n");
	printf("\t9 - Quitter l'application\n");
    printf("--------------------------------------------\n");

	if(scanf("%d", &choix) == 0)
		choix=0;

	printf("-------------------------------------------------\n");
	
	while(choix != 9){
		switch (choix) {
			case 1:
				system("clear");
				printf("\e[1;34m ██████╗ █████╗ ███╗   ██╗██████╗ ██╗██████╗  █████╗ ████████╗\n");
				printf("██╔════╝██╔══██╗████╗  ██║██╔══██╗██║██╔══██╗██╔══██╗╚══██╔══╝\n");
				printf("██║     ███████║██╔██╗ ██║██║  ██║██║██║  ██║███████║   ██║   \n");
				printf("██║     ██╔══██║██║╚██╗██║██║  ██║██║██║  ██║██╔══██║   ██║   \n");
				printf("╚██████╗██║  ██║██║ ╚████║██████╔╝██║██████╔╝██║  ██║   ██║   \n");
				printf(" ╚═════╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ ╚═╝╚═════╝ ╚═╝  ╚═╝   ╚═╝   \e[0m\n\n");

                menuCandidat(&tiut, &liste, &nbVilles, &nbCandidats);
				titreMenuPrincipal();
                break;
			case 2:
				system("clear");
				printf("\e[1;34m █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗██╗███████╗████████╗██████╗  █████╗ ████████╗███████╗██╗   ██╗██████╗ \n");
				printf("██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║██║██╔════╝╚══██╔══╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝██║   ██║██╔══██╗\n");
				printf("███████║██║  ██║██╔████╔██║██║██╔██╗ ██║██║███████╗   ██║   ██████╔╝███████║   ██║   █████╗  ██║   ██║██████╔╝\n");
				printf("██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║██║╚════██║   ██║   ██╔══██╗██╔══██║   ██║   ██╔══╝  ██║   ██║██╔══██╗\n");
				printf("██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║██║███████║   ██║   ██║  ██║██║  ██║   ██║   ███████╗╚██████╔╝██║  ██║\n");
				printf("╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝╚═╝╚══════╝   ╚═╝   ╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝ ╚═════╝ ╚═╝  ╚═╝\e[0m\n\n");

				menuAdmin(&tiut, &liste, &nbVilles, &nbCandidats);
				titreMenuPrincipal();
				break;
			case 3:
				system("clear");
				printf("\e[1;34m██████╗ ███████╗███████╗██████╗  ██████╗ ███╗   ██╗███████╗ █████╗ ██████╗ ██╗     ███████╗\n");
				printf("██╔══██╗██╔════╝██╔════╝██╔══██╗██╔═══██╗████╗  ██║██╔════╝██╔══██╗██╔══██╗██║     ██╔════╝\n");
				printf("██████╔╝█████╗  ███████╗██████╔╝██║   ██║██╔██╗ ██║███████╗███████║██████╔╝██║     █████╗  \n");
				printf("██╔══██╗██╔══╝  ╚════██║██╔═══╝ ██║   ██║██║╚██╗██║╚════██║██╔══██║██╔══██╗██║     ██╔══╝  \n");
				printf("██║  ██║███████╗███████║██║     ╚██████╔╝██║ ╚████║███████║██║  ██║██████╔╝███████╗███████╗\n");
				printf("╚═╝  ╚═╝╚══════╝╚══════╝╚═╝      ╚═════╝ ╚═╝  ╚═══╝╚══════╝╚═╝  ╚═╝╚═════╝ ╚══════╝╚══════╝\e[0m\n\n");

				menuResponsable(&tiut, &liste, &nbVilles, &nbCandidats);
				titreMenuPrincipal();
				break;
			default:
				// Permet d'aspirer les caractères en trop sur stdin (évite de répéter le message d'erreur X fois)
				while ((poubelle = getchar()) != '\n' && poubelle != EOF);
				fprintf(stderr, "\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer.\n\e[0m");
				break;
		}

		printf("--------------------------------------------\n");
		printf("Vous êtes : \n");
		printf("--------------------------------------------\n");
		printf("\t1 - Candidat\n\n");
		printf("\t2 - Administrateur\n\n");
		printf("\t3 - Responsable\n\n");
		printf("\t9 - Quitter l'application\n");
		printf("--------------------------------------------\n");
		
		if(scanf("%d", &choix) == 0)
			choix=0;
	}

	printf("\nSauvegarde des IUT...\n");
	saveVilles(tiut, nbVilles);
	printf("Sauvegarde des Candidats...\n");
	saveCandidats(liste, nbCandidats);
    printf("\nFermeture de l'application...\n");
}

VilleIUT* creerVille(void) {
	VilleIUT* ville = (VilleIUT*) malloc(sizeof(VilleIUT));

	if(ville == NULL) {
		perror("malloc");
		exit(errno);
	}

	ville->ldept = creerListeDepartement();

	return ville;
}

void ajouterVille(VilleIUT*** tiutPointer, VilleIUT* ville, int* nbVilles) {
	*tiutPointer = (VilleIUT**) realloc(*tiutPointer, (*nbVilles+1)*sizeof(VilleIUT*));

	*tiutPointer[*nbVilles] = ville;
	*nbVilles += 1;
}

int rechercherVille(VilleIUT** tiut, int nbVilles, char* searchIUT) {
	int i=0;
	for(i=0; i<nbVilles; i++)
		if(strcmp(tiut[i]->ville, searchIUT) == 0)
			return i;

	return -1;
}

void afficherListeVilles(VilleIUT** tiut, int nbVilles) {
	printf("\e[4;37mListe des villes ayant un IUT disponibles :\e[0m\n\n");

	int i;
	for(i=0; i<nbVilles; i++)
		printf("  - %s\n", tiut[i]->ville);

	printf("\n\n");
}

ListeDept creerListeDepartement(void) {
	return NULL;
}

int afficherListeDepartement(VilleIUT** tiut, int nbVilles, char* searchIUT) {
	int i = rechercherVille(tiut, nbVilles, searchIUT);

	if(i == -1) {
		fprintf(stderr, "\n\e[1;91mErreur: la ville '%s' n'est pas dans la liste des IUT.\e[0m\n\n", searchIUT);
		return -1;
	}

	ListeDept ldept = tiut[i]->ldept;

	if(ldept == NULL)
		fprintf(stderr, "\n\e[1;91mErreur: la ville '%s' ne contient aucun département\e[0m\n\n", tiut[i]->ville);
	else {
		printf("\n\e[4;37mDépartements disponibles dans la ville '%s' :\e[0m\n\n", tiut[i]->ville);
				
		while (ldept != NULL) {
			printf("  - %s, \e[0;32m%d places disponibles\e[0m\n", ldept->departement, ldept->nbP);
			ldept = ldept->suiv;
		}

		printf("\n");
	}

	return 1;
}

void afficherDepartement(VilleIUT** tiut, int nbVilles, char* searchDept) {
	printf("\n\e[4;37mListe des IUT contenant le département '%s':\e[0m\n\n", searchDept);

	int i, nb=0;
	for(i=0; i<nbVilles; i++) {
		ListeDept ldept = tiut[i]->ldept;

		if(ldept != NULL) {
			if(strcmp(ldept->departement, searchDept) == 0) {
				printf("  - %s\n", tiut[i]->ville);
				nb++;
			} else {
				ListeDept search = ldept;

				while(search->suiv != NULL) {
					search = search->suiv;
					if(strcmp(search->departement, searchDept) == 0) {
						printf("  - %s\n", tiut[i]->ville);
						nb++;
					}
				}
			}
		}
	}

	if(nb == 0)
		fprintf(stderr, "  \e[1;91mAucun IUT ne contient ce département !\e[0m\n");

	printf("\n");
}

MaillonDept* creerDepartement(char* departement, int nbP, char* responsable) {
	MaillonDept* newDept = (MaillonDept*) malloc(sizeof(MaillonDept));

	if(newDept == NULL) {
		perror("malloc");
		exit(errno);
	}

	strcpy(newDept->departement, departement);
	newDept->nbP = nbP;
	strcpy(newDept->responsable, responsable);

	return newDept;
}

ListeDept ajouterDepartement(ListeDept ldept, MaillonDept* dept) {
	ListeDept search = ldept;
	
	if(ldept == NULL)
		ldept = dept;
	else {
		while(search->suiv != NULL) 
			search = search->suiv;

		search->suiv = dept;
	}

	return ldept;
}

ListeDept supprimerDepartement(ListeDept ldept, char* searchDept) {
	int foundDept=0;

	if(strcmp(ldept->departement, searchDept) == 0) {
		ldept = ldept->suiv;
		foundDept = 1;
	} else {
		ListeDept search = ldept, tmp;

		while(search->suiv != NULL && strcmp(search->departement, searchDept) != 0) {
			tmp = search;
			search = search->suiv;
		}

		if(strcmp(search->departement, searchDept) == 0) {
			tmp->suiv = search->suiv;
			foundDept=1;
		}
	}

	if(foundDept)
		printf("\n\e[1;32mLe département '%s' a bien été supprimé !\e[0m\n\n", searchDept);
	else
		fprintf(stderr, "\n\e[1;91mErreur: la département '%s' n'existe pas.\e[0m\n\n", searchDept);

	return ldept;
}

void modifierNbPlaces(VilleIUT** tiut, int nbVilles, char* searchIUT, char* searchDept, int nb) {
	int foundDept=0, i = rechercherVille(tiut, nbVilles, searchIUT);
	
	if(i == -1) {
		fprintf(stderr, "\n\e[1;91mErreur: la ville '%s' n'est pas dans la liste des IUT.\e[0m\n\n", searchIUT);
		return;
	}

	ListeDept ldept = tiut[i]->ldept;

	if(ldept != NULL) {
		if(strcmp(ldept->departement, searchDept) == 0) {
			printf("\n\e[1;32mLe nombre de places de la formation '%s' à l'IUT '%s' est passé de %d à %d\e[0m\n\n", searchDept, searchIUT, ldept->nbP, nb);
			ldept->nbP = nb;
			foundDept = 1;
		} else {
			ListeDept search = ldept;

			while(search->suiv != NULL) {
				if(strcmp(search->departement, searchDept) == 0) {
					printf("\n\e[1;32mLe nombre de places de la formation '%s' à l'IUT '%s' est passé de %d à %d\e[0m\n\n", searchDept, searchIUT, search->nbP, nb);
					search->nbP = nb;
					foundDept = 1;
				}
				search = search->suiv;
			}
		}
	}

	if(!foundDept)
		fprintf(stderr, "\n\e[1;91mErreur: le département '%s' n'a pas été trouvé dans l'IUT '%s'\e[0m\n\n", searchDept, searchIUT);
}

void modifierNomResponsable(VilleIUT** tiut, int nbVilles, char* searchIUT, char* searchDept, char* nom) {
	int foundDept=0, i = rechercherVille(tiut, nbVilles, searchIUT);

	if(i == -1) {
		fprintf(stderr, "\n\e[1;91mErreur: la ville '%s' n'est pas dans la liste des IUT.\e[0m\n\n", searchIUT);
		return;
	}

	ListeDept ldept = tiut[i]->ldept;

	if(ldept != NULL) {
		if(strcmp(ldept->departement, searchDept) == 0) {
			strcpy(ldept->responsable, nom);
			printf("\n\e[1;32mLe responsable de la formation '%s' à l'IUT '%s' est désormais '%s'.\e[0m\n\n", searchDept, searchIUT, ldept->responsable);
			foundDept = 1;
		} else {
			ListeDept search = ldept;

			while(search->suiv != NULL) {
				search = search->suiv;
				if(strcmp(search->departement, searchDept) == 0) {
					strcpy(search->responsable, nom);
					printf("\n\e[1;32mLe responsable de la formation '%s' à l'IUT '%s' est désormais '%s'.\e[0m\n\n", searchDept, searchIUT, search->responsable);
					foundDept = 1;
				}
			}
		}
	}

	if(foundDept == 0)
		fprintf(stderr, "\n\e[1;91mErreur: le département '%s' n'a pas été trouvé dans l'IUT '%s'.\e[0m\n\n", searchDept, searchIUT);
}

void saveVilles(VilleIUT* tiut[], int nbVilles) {
    FILE *fe = fopen("donnees/villes.bin", "wb");

    if(fe == NULL) {
        fprintf(stderr, "\e[1;91mErreur: impossible d'écrire dans le fichier villes.don\e[0m");
        return;
    }

	fwrite(&nbVilles, sizeof(int), 1, fe);

	for (int i = 0; i < nbVilles; i++) {
		VilleIUT ville = *tiut[i];

		int longueurNomVille = strlen(ville.ville)+1;
		fwrite(&longueurNomVille, sizeof(int), 1, fe);
		fwrite(ville.ville, sizeof(char), longueurNomVille, fe);

		int nbDepartements = 0;
		MaillonDept* maillon = ville.ldept;
		while (maillon != NULL) {
			nbDepartements++;
			maillon = maillon->suiv;
		}
		fwrite(&nbDepartements, sizeof(int), 1, fe);

		maillon = ville.ldept;
		while (maillon != NULL) {
			int lenDepartement = strlen(maillon->departement)+1;
			fwrite(&lenDepartement, sizeof(int), 1, fe);
			fwrite(maillon->departement, sizeof(char), lenDepartement, fe);

			fwrite(&maillon->nbP, sizeof(int), 1, fe);

			int lenNomResponsable = strlen(maillon->responsable)+1;
			fwrite(&lenNomResponsable, sizeof(int), 1, fe);
			fwrite(maillon->responsable, sizeof(char), lenNomResponsable, fe);

			maillon = maillon->suiv;
		}
	}

    fclose(fe);
}

VilleIUT** readVilles(int* nbVilles) {
	FILE *fe = fopen("donnees/villes.bin", "rb");

    if(fe == NULL) {
        fprintf(stderr, "\e[1;91mErreur: impossible de lire le fichier villes.bin\e[0m");
        exit(1);
    }

	fread(nbVilles, sizeof(int), 1, fe);

	VilleIUT** tiut = (VilleIUT**) malloc(*nbVilles*sizeof(VilleIUT*));

	if(tiut == NULL) {
		perror("malloc");
		exit(errno);
	}

	for (int i = 0; i < *nbVilles; i++) {
		VilleIUT* ville = (VilleIUT*) malloc(sizeof(VilleIUT));

		if(ville == NULL) {
			perror("malloc");
			exit(errno);
		}

		int lenNomVille;
		fread(&lenNomVille, sizeof(int), 1, fe);
		fread(ville->ville, sizeof(char), lenNomVille, fe);

		int nbDepartements;
		fread(&nbDepartements, sizeof(int), 1, fe);

		ville->ldept = creerListeDepartement();
		MaillonDept* maillonPrecedent = NULL;

		int j;
		for (j=0; j<nbDepartements; j++) {
			MaillonDept* maillon = (MaillonDept*) malloc(sizeof(MaillonDept));

			if(maillon == NULL) {
				perror("malloc");
				exit(errno);
			}
			
			if(maillonPrecedent != NULL)
				maillonPrecedent->suiv = maillon;

			int lenDepartement;
			fread(&lenDepartement, sizeof(int), 1, fe);
			fread(maillon->departement, sizeof(char), lenDepartement, fe);

			fread(&maillon->nbP, sizeof(int), 1, fe);

			int lenNomResponsable;
			fread(&lenNomResponsable, sizeof(int), 1, fe);
			fread(maillon->responsable, sizeof(char), lenNomResponsable, fe);
			
			if(maillonPrecedent == NULL)
				ville->ldept = maillon;

			maillonPrecedent = maillon;
		}

		tiut[i] = ville;
	}

    fclose(fe);

    return tiut;
}

void test(void) {
	/*
	Ajout des premières données
	*/
	VilleIUT** tiut = (VilleIUT**) malloc(sizeof(VilleIUT*));

	if(tiut == NULL) {
		perror("malloc");
		exit(errno);
	}

	int nbVilles = 0;

	VilleIUT* clermont = creerVille();
	strcpy(clermont->ville, "Clermont-Ferrand");

	ajouterVille(&tiut, clermont, &nbVilles);

	//afficherListeDepartement(*clermont);

	MaillonDept* departement = creerDepartement("Informatique", 136, "Simon Carine");
	tiut[0]->ldept = ajouterDepartement(tiut[0]->ldept, departement);

	departement = creerDepartement("Biologie", 120, "IDK");
	tiut[0]->ldept = ajouterDepartement(tiut[0]->ldept, departement);

	departement = creerDepartement("Testeuuu", 1337, "IDK");
	tiut[0]->ldept = ajouterDepartement(tiut[0]->ldept, departement);

	//afficherListeDepartement(*clermont);

	saveVilles(tiut, nbVilles);

	// BINAIRE

	VilleIUT** tiut2;
	int nbVilles2;

	tiut2 = readVilles(&nbVilles2);

	//afficherListeDepartement(*tiut2[0]);

	VilleIUT* grenoble = creerVille();
	strcpy(grenoble->ville, "Grenoble");

	ajouterVille(&tiut2, grenoble, &nbVilles2);

	departement = creerDepartement("Informatique", 1426, "Simonette Caro");

	tiut2[1]->ldept = ajouterDepartement(tiut2[1]->ldept, departement);

	afficherListeVilles(tiut2, nbVilles2);

	afficherDepartement(tiut2, nbVilles2, "CACA");

	tiut2[0]->ldept = supprimerDepartement(tiut2[0]->ldept, "Testeuuu");

	//afficherListeDepartement(*tiut2[0]);

	modifierNbPlaces(tiut2, nbVilles2, "Clermond Ft", "Informatique", 140);

	//afficherListeDepartement(*tiut2[1]);

	//modifierNomResponsable(tiut2, nbVilles2);
}

/*
int chargementDonnees (VilleIUT *tiut[]){
    int i=0;
    FILE *fe;
    if ((fe = fopen("iut.don","r"))==NULL){
        fprintf(stderr, "Impossible d'ouvrir le fichier");
        return -1;
    }
    
    tiut[i] = (VilleIUT *)malloc(20*sizeof(VilleIUT));
    if (tiut == NULL){
        printf("\n Pb sur malloc\n");
        exit(1);
    }

    fscanf(fe, "%s", tiut[i]->ville);
	fscanf(fe, "%d", tiut[i]->nbDep);
	ListDept *l = creerListe();
	MaillonDept maillon = (MaillonDept)malloc(sizeof(MaillonDept))
	fscanf(fe, "%s\t%d\t%s", maillon -> departement, &maillon -> nbP, maillon -> responsable);
	maillon -> suivant = NULL;
	if (l->premier != NULL){
		MaillonDept dep = l->premier;
		while (dep -> suivant != NULL){
			dep = dep->suivant;

		} 	
	}
	return i;
}*/
